@@ -1,20 +1,19 @@
-net = require('net');
-connections = 0;
+'use strict';
 
+const net = require('net');
+var connections = 0;
 var errors = 0;
 
-server = net.Server(function (socket) {
-
-  socket.on('end', function () {
-    socket.end();
-  });
+var server = net.Server(function (socket) {
 
   socket.on('error', function () {
-    errors++; 
+    errors++;
   });
 
 });
 
+//server.maxConnections = 128;
+
 server.listen(9000);
 
 var oldConnections, oldErrors;
@@ -30,4 +29,3 @@ setInterval(function () {
     console.log("SERVER %d errors: %d", process.pid, errors);
   }
 }, 1000);
-
