@@ -5,14 +5,17 @@
 #ifndef V8_HEAP_PROFILER_H_
 #define V8_HEAP_PROFILER_H_
 
-#include "src/heap-snapshot-generator-inl.h"
+#include "src/base/smart-pointers.h"
 #include "src/isolate.h"
-#include "src/smart-pointers.h"
 
 namespace v8 {
 namespace internal {
 
+// Forward declarations.
+class AllocationTracker;
+class HeapObjectsMap;
 class HeapSnapshot;
+class StringsStorage;
 
 class HeapProfiler {
  public:
@@ -22,11 +25,6 @@ class HeapProfiler {
   size_t GetMemorySizeUsedByProfiler();
 
   HeapSnapshot* TakeSnapshot(
-      const char* name,
-      v8::ActivityControl* control,
-      v8::HeapProfiler::ObjectNameResolver* resolver);
-  HeapSnapshot* TakeSnapshot(
-      String* name,
       v8::ActivityControl* control,
       v8::HeapProfiler::ObjectNameResolver* resolver);
 
@@ -38,7 +36,8 @@ class HeapProfiler {
   HeapObjectsMap* heap_object_map() const { return ids_.get(); }
   StringsStorage* names() const { return names_.get(); }
 
-  SnapshotObjectId PushHeapObjectsStats(OutputStream* stream);
+  SnapshotObjectId PushHeapObjectsStats(OutputStream* stream,
+                                        int64_t* timestamp_us);
   int GetSnapshotsCount();
   HeapSnapshot* GetSnapshot(int index);
   SnapshotObjectId GetSnapshotObjectId(Handle<Object> obj);
@@ -67,15 +66,14 @@ class HeapProfiler {
   void ClearHeapObjectMap();
 
  private:
-  Heap* heap() const { return ids_->heap(); }
+  Heap* heap() const;
 
   // Mapping from HeapObject addresses to objects' uids.
-  SmartPointer<HeapObjectsMap> ids_;
+  base::SmartPointer<HeapObjectsMap> ids_;
   List<HeapSnapshot*> snapshots_;
-  SmartPointer<StringsStorage> names_;
-  unsigned next_snapshot_uid_;
+  base::SmartPointer<StringsStorage> names_;
   List<v8::HeapProfiler::WrapperInfoCallback> wrapper_callbacks_;
-  SmartPointer<AllocationTracker> allocation_tracker_;
+  base::SmartPointer<AllocationTracker> allocation_tracker_;
   bool is_tracking_object_moves_;
 };
 
