@@ -1,32 +1,15 @@
-// Copyright Joyent, Inc. and other Node contributors.
-//
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the
-// "Software"), to deal in the Software without restriction, including
-// without limitation the rights to use, copy, modify, merge, publish,
-// distribute, sublicense, and/or sell copies of the Software, and to permit
-// persons to whom the Software is furnished to do so, subject to the
-// following conditions:
-//
-// The above copyright notice and this permission notice shall be included
-// in all copies or substantial portions of the Software.
-//
-// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
-// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
-// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
-// USE OR OTHER DEALINGS IN THE SOFTWARE.
+'use strict';
 
-var util = require('util'),
-    path = require('path'),
-    net = require('net'),
-    vm = require('vm'),
-    repl = require('repl'),
-    inherits = util.inherits,
-    assert = require('assert'),
-    spawn = require('child_process').spawn;
+const util = require('util');
+const path = require('path');
+const net = require('net');
+const vm = require('vm');
+const Module = require('module');
+const repl = require('repl');
+const inherits = util.inherits;
+const assert = require('assert');
+const spawn = require('child_process').spawn;
+const Buffer = require('buffer').Buffer;
 
 exports.start = function(argv, stdin, stdout) {
   argv || (argv = process.argv.slice(2));
@@ -42,8 +25,8 @@ exports.start = function(argv, stdin, stdout) {
   stdin = stdin || process.stdin;
   stdout = stdout || process.stdout;
 
-  var args = ['--debug-brk'].concat(argv),
-      interface_ = new Interface(stdin, stdout, args);
+  const args = ['--debug-brk'].concat(argv);
+  const interface_ = new Interface(stdin, stdout, args);
 
   stdin.resume();
 
@@ -62,7 +45,7 @@ exports.port = 5858;
 
 //
 // Parser/Serializer for V8 debugger protocol
-// http://code.google.com/p/v8/wiki/DebuggerProtocol
+// https://github.com/v8/v8/wiki/Debugging-Protocol
 //
 // Usage:
 //    p = new Protocol();
@@ -119,7 +102,7 @@ Protocol.prototype.execute = function(d) {
       if (len - this.bodyStartByteIndex < this.contentLength) {
         break;
       }
-      // pass thru
+      // falls through
     case 'body':
       var resRawByteLength = Buffer.byteLength(res.raw, 'utf8');
 
@@ -143,7 +126,6 @@ Protocol.prototype.execute = function(d) {
 
     default:
       throw new Error('Unknown state');
-      break;
   }
 };
 
@@ -157,10 +139,10 @@ Protocol.prototype.serialize = function(req) {
 };
 
 
-var NO_FRAME = -1;
+const NO_FRAME = -1;
 
 function Client() {
-  net.Stream.call(this);
+  net.Socket.call(this);
   var protocol = this.protocol = new Protocol(this);
   this._reqCallbacks = [];
   var socket = this;
@@ -177,26 +159,27 @@ function Client() {
     protocol.execute(d);
   });
 
-  protocol.onResponse = this._onResponse.bind(this);
+  protocol.onResponse = (res) => this._onResponse(res);
 }
-inherits(Client, net.Stream);
+inherits(Client, net.Socket);
 exports.Client = Client;
 
 
 Client.prototype._addHandle = function(desc) {
-  if (!util.isObject(desc) || !util.isNumber(desc.handle)) {
+  if (desc === null || typeof desc !== 'object' ||
+      typeof desc.handle !== 'number') {
     return;
   }
 
   this.handles[desc.handle] = desc;
 
-  if (desc.type == 'script') {
+  if (desc.type === 'script') {
     this._addScript(desc);
   }
 };
 
 
-var natives = process.binding('natives');
+const natives = process.binding('natives');
 
 
 Client.prototype._addScript = function(desc) {
@@ -214,8 +197,8 @@ Client.prototype._removeScript = function(desc) {
 
 
 Client.prototype._onResponse = function(res) {
-  var cb,
-      index = -1;
+  var cb;
+  var index = -1;
 
   this._reqCallbacks.some(function(fn, i) {
     if (fn.request_seq == res.body.request_seq) {
@@ -279,7 +262,7 @@ Client.prototype.req = function(req, cb) {
 
 Client.prototype.reqVersion = function(cb) {
   cb = cb || function() {};
-  this.req({ command: 'version' } , function(err, body, res) {
+  this.req({ command: 'version' }, function(err, body, res) {
     if (err) return cb(err);
     cb(null, res.body.body.V8Version, res.body.running);
   });
@@ -302,7 +285,7 @@ Client.prototype.reqLookup = function(refs, cb) {
   this.req(req, function(err, res) {
     if (err) return cb(err);
     for (var ref in res) {
-      if (util.isObject(res[ref])) {
+      if (res[ref] !== null && typeof res[ref] === 'object') {
         self._addHandle(res[ref]);
       }
     }
@@ -312,11 +295,11 @@ Client.prototype.reqLookup = function(refs, cb) {
 };
 
 Client.prototype.reqScopes = function(cb) {
-  var self = this,
-      req = {
-        command: 'scopes',
-        arguments: {}
-      };
+  const self = this;
+  const req = {
+    command: 'scopes',
+    arguments: {}
+  };
 
   cb = cb || function() {};
   this.req(req, function(err, res) {
@@ -415,7 +398,7 @@ Client.prototype.reqFrameEval = function(expression, frame, cb) {
 // reqBacktrace(cb)
 // TODO: from, to, bottom
 Client.prototype.reqBacktrace = function(cb) {
-  this.req({ command: 'backtrace', arguments: { inlineRefs: true } } , cb);
+  this.req({ command: 'backtrace', arguments: { inlineRefs: true } }, cb);
 };
 
 
@@ -449,7 +432,7 @@ Client.prototype.reqScripts = function(cb) {
   var self = this;
   cb = cb || function() {};
 
-  this.req({ command: 'scripts' } , function(err, res) {
+  this.req({ command: 'scripts' }, function(err, res) {
     if (err) return cb(err);
 
     for (var i = 0; i < res.length; i++) {
@@ -479,7 +462,7 @@ Client.prototype.setBreakpoint = function(req, cb) {
 };
 
 Client.prototype.clearBreakpoint = function(req, cb) {
-  var req = {
+  req = {
     command: 'clearbreakpoint',
     arguments: req
   };
@@ -542,8 +525,8 @@ Client.prototype.mirrorObject = function(handle, depth, cb) {
         return;
       }
 
-      var mirror,
-          waiting = 1;
+      var mirror;
+      var waiting = 1;
 
       if (handle.className == 'Array') {
         mirror = [];
@@ -564,8 +547,7 @@ Client.prototype.mirrorObject = function(handle, depth, cb) {
           mirrorValue = '[?]';
         }
 
-
-        if (util.isArray(mirror) && !util.isNumber(prop.name)) {
+        if (Array.isArray(mirror) && typeof prop.name !== 'number') {
           // Skip the 'length' property.
           return;
         }
@@ -585,29 +567,27 @@ Client.prototype.mirrorObject = function(handle, depth, cb) {
 
       waitForOthers();
       function waitForOthers() {
-        if (--waiting === 0 && cb) {
+        if (--waiting === 0) {
           keyValues.forEach(function(kv) {
             mirror[kv.name] = kv.value;
           });
           cb(null, mirror);
         }
-      };
+      }
     });
     return;
   } else if (handle.type === 'function') {
     val = function() {};
   } else if (handle.type === 'null') {
     val = null;
-  } else if (!util.isUndefined(handle.value)) {
+  } else if (handle.value !== undefined) {
     val = handle.value;
   } else if (handle.type === 'undefined') {
     val = undefined;
   } else {
     val = handle;
   }
-  process.nextTick(function() {
-    cb(null, val);
-  });
+  process.nextTick(cb, null, val);
 };
 
 
@@ -661,11 +641,7 @@ Client.prototype.fullTrace = function(cb) {
 };
 
 
-
-
-
-
-var commands = [
+const commands = [
   [
     'run (r)',
     'cont (c)',
@@ -681,6 +657,7 @@ var commands = [
     'unwatch',
     'watchers',
     'repl',
+    'exec',
     'restart',
     'kill',
     'list',
@@ -700,8 +677,8 @@ var helpMessage = 'Commands: ' + commands.map(function(group) {
 function SourceUnderline(sourceText, position, repl) {
   if (!sourceText) return '';
 
-  var head = sourceText.slice(0, position),
-      tail = sourceText.slice(position);
+  const head = sourceText.slice(0, position);
+  var tail = sourceText.slice(position);
 
   // Colourize char if stdout supports colours
   if (repl.useColors) {
@@ -721,8 +698,8 @@ function SourceInfo(body) {
 
   if (body.script) {
     if (body.script.name) {
-      var name = body.script.name,
-          dir = path.resolve() + '/';
+      var name = body.script.name;
+      const dir = path.resolve() + '/';
 
       // Change path to relative, if possible
       if (name.indexOf(dir) === 0) {
@@ -757,7 +734,7 @@ function Interface(stdin, stdout, args) {
     prompt: 'debug> ',
     input: this.stdin,
     output: this.stdout,
-    eval: this.controlEval.bind(this),
+    eval: (code, ctx, file, cb) => this.controlEval(code, ctx, file, cb),
     useGlobal: false,
     ignoreUndefined: true
   };
@@ -776,6 +753,7 @@ function Interface(stdin, stdout, args) {
   if (parseInt(process.env['NODE_DISABLE_COLORS'], 10)) {
     opts.useColors = false;
   }
+
   this.repl = repl.start(opts);
 
   // Do not print useless warning
@@ -787,26 +765,26 @@ function Interface(stdin, stdout, args) {
   });
 
   // Handle all possible exits
-  process.on('exit', this.killChild.bind(this));
+  process.on('exit', () => this.killChild());
   process.once('SIGTERM', process.exit.bind(process, 0));
   process.once('SIGHUP', process.exit.bind(process, 0));
 
-  var proto = Interface.prototype,
-      ignored = ['pause', 'resume', 'exitRepl', 'handleBreak',
-                 'requireConnection', 'killChild', 'trySpawn',
-                 'controlEval', 'debugEval', 'print', 'childPrint',
-                 'clearline'],
-      shortcut = {
-        'run': 'r',
-        'cont': 'c',
-        'next': 'n',
-        'step': 's',
-        'out': 'o',
-        'backtrace': 'bt',
-        'setBreakpoint': 'sb',
-        'clearBreakpoint': 'cb',
-        'pause_': 'pause'
-      };
+  var proto = Interface.prototype;
+  const ignored = ['pause', 'resume', 'exitRepl', 'handleBreak',
+                   'requireConnection', 'killChild', 'trySpawn',
+                   'controlEval', 'debugEval', 'print', 'childPrint',
+                   'clearline'];
+  const shortcut = {
+    'run': 'r',
+    'cont': 'c',
+    'next': 'n',
+    'step': 's',
+    'out': 'o',
+    'backtrace': 'bt',
+    'setBreakpoint': 'sb',
+    'clearBreakpoint': 'cb',
+    'pause_': 'pause'
+  };
 
   function defineProperty(key, protoKey) {
     // Check arity
@@ -821,7 +799,7 @@ function Interface(stdin, stdout, args) {
     } else {
       self.repl.context[key] = fn;
     }
-  };
+  }
 
   // Copy all prototype methods in repl context
   // Setup them as getters if possible
@@ -898,7 +876,7 @@ Interface.prototype.print = function(text, oneline) {
   if (this.killed) return;
   this.clearline();
 
-  this.stdout.write(util.isString(text) ? text : util.inspect(text));
+  this.stdout.write(typeof text === 'string' ? text : util.inspect(text));
 
   if (oneline !== true) {
     this.stdout.write('\n');
@@ -972,6 +950,12 @@ Interface.prototype.controlEval = function(code, context, filename, callback) {
       }
     }
 
+    // exec process.title => exec("process.title");
+    var match = code.match(/^\s*exec\s+([^\n]*)/);
+    if (match) {
+      code = 'exec(' + JSON.stringify(match[1]) + ')';
+    }
+
     var result = vm.runInContext(code, context, filename);
 
     // Repl should not ask for next command
@@ -992,8 +976,8 @@ Interface.prototype.controlEval = function(code, context, filename, callback) {
 Interface.prototype.debugEval = function(code, context, filename, callback) {
   if (!this.requireConnection()) return;
 
-  var self = this,
-      client = this.client;
+  const self = this;
+  const client = this.client;
 
   // Repl asked for scope variables
   if (code === '.scope') {
@@ -1027,9 +1011,9 @@ Interface.prototype.debugEval = function(code, context, filename, callback) {
 // Adds spaces and prefix to number
 // maxN is a maximum number we should have space for
 function leftPad(n, prefix, maxN) {
-  var s = n.toString(),
-      nchars = Math.max(2, String(maxN).length) + 1,
-      nspaces = nchars - s.length - 1;
+  const s = n.toString();
+  const nchars = Math.max(2, String(maxN).length) + 1;
+  const nspaces = nchars - s.length - 1;
 
   for (var i = 0; i < nspaces; i++) {
     prefix += ' ';
@@ -1101,10 +1085,10 @@ Interface.prototype.list = function(delta) {
 
   delta || (delta = 5);
 
-  var self = this,
-      client = this.client,
-      from = client.currentSourceLine - delta + 1,
-      to = client.currentSourceLine + delta + 1;
+  const self = this;
+  const client = this.client;
+  const from = client.currentSourceLine - delta + 1;
+  const to = client.currentSourceLine + delta + 1;
 
   self.pause();
   client.reqSource(from, to, function(err, res) {
@@ -1119,17 +1103,17 @@ Interface.prototype.list = function(delta) {
       var lineno = res.fromLine + i + 1;
       if (lineno < from || lineno > to) continue;
 
-      var current = lineno == 1 + client.currentSourceLine,
-          breakpoint = client.breakpoints.some(function(bp) {
-            return (bp.scriptReq === client.currentScript ||
-                    bp.script === client.currentScript) &&
-                    bp.line == lineno;
-          });
+      const current = lineno == 1 + client.currentSourceLine;
+      const breakpoint = client.breakpoints.some(function(bp) {
+        return (bp.scriptReq === client.currentScript ||
+                bp.script === client.currentScript) &&
+                bp.line == lineno;
+      });
 
       if (lineno == 1) {
         // The first line needs to have the module wrapper filtered out of
         // it.
-        var wrapper = require('module').wrapper[0];
+        var wrapper = Module.wrapper[0];
         lines[i] = lines[i].slice(wrapper.length);
 
         client.currentSourceColumn -= wrapper.length;
@@ -1162,8 +1146,8 @@ Interface.prototype.list = function(delta) {
 Interface.prototype.backtrace = function() {
   if (!this.requireConnection()) return;
 
-  var self = this,
-      client = this.client;
+  const self = this;
+  const client = this.client;
 
   self.pause();
   client.fullTrace(function(err, bt) {
@@ -1176,8 +1160,8 @@ Interface.prototype.backtrace = function() {
     if (bt.totalFrames == 0) {
       self.print('(empty stack)');
     } else {
-      var trace = [],
-          firstFrameNative = bt.frames[0].script.isNative;
+      const trace = [];
+      const firstFrameNative = bt.frames[0].script.isNative;
 
       for (var i = 0; i < bt.frames.length; i++) {
         var frame = bt.frames[i];
@@ -1206,14 +1190,14 @@ Interface.prototype.backtrace = function() {
 Interface.prototype.scripts = function() {
   if (!this.requireConnection()) return;
 
-  var client = this.client,
-      displayNatives = arguments[0] || false,
-      scripts = [];
+  const client = this.client;
+  const displayNatives = arguments[0] || false;
+  const scripts = [];
 
   this.pause();
   for (var id in client.scripts) {
     var script = client.scripts[id];
-    if (util.isObject(script) && script.name) {
+    if (script !== null && typeof script === 'object' && script.name) {
       if (displayNatives ||
           script.name == client.currentScript ||
           !script.isNative) {
@@ -1288,11 +1272,11 @@ Interface.prototype.unwatch = function(expr) {
 
 // List watchers
 Interface.prototype.watchers = function() {
-  var self = this,
-      verbose = arguments[0] || false,
-      callback = arguments[1] || function() {},
-      waiting = this._watchers.length,
-      values = [];
+  var self = this;
+  var verbose = arguments[0] || false;
+  var callback = arguments[1] || function() {};
+  var waiting = this._watchers.length;
+  var values = [];
 
   this.pause();
 
@@ -1346,18 +1330,18 @@ Interface.prototype.setBreakpoint = function(script, line,
                                              condition, silent) {
   if (!this.requireConnection()) return;
 
-  var self = this,
-      scriptId,
-      ambiguous;
+  const self = this;
+  var scriptId;
+  var ambiguous;
 
   // setBreakpoint() should insert breakpoint on current line
-  if (util.isUndefined(script)) {
+  if (script === undefined) {
     script = this.client.currentScript;
     line = this.client.currentSourceLine + 1;
   }
 
   // setBreakpoint(line-number) should insert breakpoint in current script
-  if (util.isUndefined(line) && util.isNumber(script)) {
+  if (line === undefined && typeof script === 'number') {
     line = script;
     script = this.client.currentScript;
   }
@@ -1368,9 +1352,10 @@ Interface.prototype.setBreakpoint = function(script, line,
     return;
   }
 
+  let req;
   if (/\(\)$/.test(script)) {
     // setBreakpoint('functionname()');
-    var req = {
+    req = {
       type: 'function',
       target: script.replace(/\(\)$/, ''),
       condition: condition
@@ -1379,9 +1364,7 @@ Interface.prototype.setBreakpoint = function(script, line,
     // setBreakpoint('scriptname')
     if (script != +script && !this.client.scripts[script]) {
       var scripts = this.client.scripts;
-      var keys = Object.keys(scripts);
-      for (var v = 0; v < keys.length; v++) {
-        var id = keys[v];
+      for (var id in scripts) {
         if (scripts[id] &&
             scripts[id].name &&
             scripts[id].name.indexOf(script) !== -1) {
@@ -1398,7 +1381,6 @@ Interface.prototype.setBreakpoint = function(script, line,
     if (ambiguous) return this.error('Script name is ambiguous');
     if (line <= 0) return this.error('Line should be a positive value');
 
-    var req;
     if (scriptId) {
       req = {
         type: 'scriptId',
@@ -1454,17 +1436,19 @@ Interface.prototype.setBreakpoint = function(script, line,
 Interface.prototype.clearBreakpoint = function(script, line) {
   if (!this.requireConnection()) return;
 
-  var ambiguous,
-      breakpoint,
-      index;
+  var ambiguous;
+  var breakpoint;
+  var scriptId;
+  var index;
 
   this.client.breakpoints.some(function(bp, i) {
     if (bp.scriptId === script ||
         bp.scriptReq === script ||
         (bp.script && bp.script.indexOf(script) !== -1)) {
-      if (!util.isUndefined(index)) {
+      if (index !== undefined) {
         ambiguous = true;
       }
+      scriptId = script;
       if (bp.line === line) {
         index = i;
         breakpoint = bp.id;
@@ -1473,16 +1457,32 @@ Interface.prototype.clearBreakpoint = function(script, line) {
     }
   });
 
-  if (ambiguous) return this.error('Script name is ambiguous');
-
-  if (util.isUndefined(breakpoint)) {
-    return this.error('Script : ' + script + ' not found');
+  if (!scriptId && !this.client.scripts[script]) {
+    var scripts = this.client.scripts;
+    for (var id in scripts) {
+      if (scripts[id] &&
+          scripts[id].name &&
+          scripts[id].name.indexOf(script) !== -1) {
+        if (scriptId) {
+          ambiguous = true;
+        }
+        scriptId = id;
+      }
+    }
   }
 
-  var self = this,
-      req = {
-        breakpoint: breakpoint
-      };
+  if (ambiguous) return this.error('Script name is ambiguous');
+
+  if (scriptId === undefined) {
+    return this.error('Script ' + script + ' not found');
+  }
+
+  if (breakpoint === undefined) {
+    return this.error('Breakpoint not found on line ' + line);
+  }
+
+  var self = this;
+  const req = {breakpoint};
 
   self.pause();
   self.client.clearBreakpoint(req, function(err, res) {
@@ -1518,8 +1518,8 @@ Interface.prototype.breakpoints = function() {
 Interface.prototype.pause_ = function() {
   if (!this.requireConnection()) return;
 
-  var self = this,
-      cmd = 'process._debugPause();';
+  const self = this;
+  const cmd = 'process._debugPause();';
 
   this.pause();
   this.client.reqFrameEval(cmd, NO_FRAME, function(err, res) {
@@ -1532,6 +1532,18 @@ Interface.prototype.pause_ = function() {
 };
 
 
+// execute expression
+Interface.prototype.exec = function(code) {
+  this.debugEval(code, null, null, (err, result) => {
+    if (err) {
+      this.error(err);
+    } else {
+      this.print(util.inspect(result, {colors: true}));
+    }
+  });
+};
+
+
 // Kill child process
 Interface.prototype.kill = function() {
   if (!this.child) return;
@@ -1551,8 +1563,7 @@ Interface.prototype.repl = function() {
   var listeners = this.repl.rli.listeners('SIGINT').slice(0);
   this.repl.rli.removeAllListeners('SIGINT');
 
-  // Exit debug repl on Ctrl + C
-  this.repl.rli.once('SIGINT', function() {
+  function exitDebugRepl() {
     // Restore all listeners
     process.nextTick(function() {
       listeners.forEach(function(listener) {
@@ -1562,10 +1573,20 @@ Interface.prototype.repl = function() {
 
     // Exit debug repl
     self.exitRepl();
-  });
+
+    self.repl.rli.removeListener('SIGINT', exitDebugRepl);
+    self.repl.removeListener('exit', exitDebugRepl);
+  }
+
+  // Exit debug repl on SIGINT
+  this.repl.rli.on('SIGINT', exitDebugRepl);
+
+  // Exit debug repl on repl exit
+  this.repl.on('exit', exitDebugRepl);
 
   // Set new
-  this.repl.eval = this.debugEval.bind(this);
+  this.repl.eval = (code, ctx, file, cb) =>
+    this.debugEval(code, ctx, file, cb);
   this.repl.context = {};
 
   // Swap history
@@ -1580,7 +1601,8 @@ Interface.prototype.repl = function() {
 // Exit debug repl
 Interface.prototype.exitRepl = function() {
   // Restore eval
-  this.repl.eval = this.controlEval.bind(this);
+  this.repl.eval = (code, ctx, file, cb) =>
+    this.controlEval(code, ctx, file, cb);
 
   // Swap history
   this.history.debug = this.repl.rli.history;
@@ -1618,18 +1640,18 @@ Interface.prototype.killChild = function() {
 
 // Spawns child process (and restores breakpoints)
 Interface.prototype.trySpawn = function(cb) {
-  var self = this,
-      breakpoints = this.breakpoints || [],
-      port = exports.port,
-      host = 'localhost',
-      childArgs = this.args;
+  const self = this;
+  const breakpoints = this.breakpoints || [];
+  var port = exports.port;
+  var host = '127.0.0.1';
+  var childArgs = this.args;
 
   this.killChild();
   assert(!this.child);
 
   var isRemote = false;
   if (this.args.length === 2) {
-    var match = this.args[1].match(/^([^:]+):(\d+)$/);
+    const match = this.args[1].match(/^([^:]+):(\d+)$/);
 
     if (match) {
       // Connecting to remote debugger
@@ -1641,10 +1663,19 @@ Interface.prototype.trySpawn = function(cb) {
   } else if (this.args.length === 3) {
     // `node debug -p pid`
     if (this.args[1] === '-p' && /^\d+$/.test(this.args[2])) {
-      process._debugProcess(parseInt(this.args[2], 10));
+      const pid = parseInt(this.args[2], 10);
+      try {
+        process._debugProcess(pid);
+      } catch (e) {
+        if (e.code === 'ESRCH') {
+          console.error(`Target process: ${pid} doesn't exist.`);
+          process.exit(1);
+        }
+        throw e;
+      }
       isRemote = true;
     } else {
-      var match = this.args[1].match(/^--port=(\d+)$/);
+      const match = this.args[1].match(/^--port=(\d+)$/);
       if (match) {
         // Start debugger on custom port
         // `node debug --port=5858 app.js`
@@ -1658,14 +1689,14 @@ Interface.prototype.trySpawn = function(cb) {
     // pipe stream into debugger
     this.child = spawn(process.execPath, childArgs);
 
-    this.child.stdout.on('data', this.childPrint.bind(this));
-    this.child.stderr.on('data', this.childPrint.bind(this));
+    this.child.stdout.on('data', (text) => this.childPrint(text));
+    this.child.stderr.on('data', (text) => this.childPrint(text));
   }
 
   this.pause();
 
-  var client = self.client = new Client(),
-      connectionAttempts = 0;
+  const client = self.client = new Client();
+  var connectionAttempts = 0;
 
   client.once('ready', function() {
     self.stdout.write(' ok\n');
@@ -1706,8 +1737,8 @@ Interface.prototype.trySpawn = function(cb) {
   function connectError() {
     // If it's failed to connect 10 times then print failed message
     if (connectionAttempts >= 10) {
-      self.stdout.write(' failed, please retry\n');
-      return;
+      console.error(' failed, please retry');
+      process.exit(1);
     }
     setTimeout(attemptConnect, 500);
   }
@@ -1718,11 +1749,12 @@ Interface.prototype.trySpawn = function(cb) {
     client.connect(port, host);
   }
 
-  self.print('connecting to ' + host + ':' + port + ' ..', true);
   if (isRemote) {
+    self.print('connecting to ' + host + ':' + port + ' ..', true);
     attemptConnect();
   } else {
     this.child.stderr.once('data', function() {
+      self.print('connecting to ' + host + ':' + port + ' ..', true);
       setImmediate(attemptConnect);
     });
   }
