@@ -189,6 +189,9 @@ exports.lookupService = function(host, port, callback) {
   if (cares.isIP(host) === 0)
     throw new TypeError('host needs to be a valid IP address');
 
+  if (typeof port !== 'number')
+    throw new TypeError(`port argument must be a number, got "${port}"`);
+
   callback = makeAsync(callback);
 
   var req = new GetNameInfoReqWrap();
@@ -330,7 +333,6 @@ exports.NOTFOUND = 'ENOTFOUND';
 exports.NOTIMP = 'ENOTIMP';
 exports.REFUSED = 'EREFUSED';
 exports.BADQUERY = 'EBADQUERY';
-exports.ADNAME = 'EADNAME';
 exports.BADNAME = 'EBADNAME';
 exports.BADFAMILY = 'EBADFAMILY';
 exports.BADRESP = 'EBADRESP';
