@@ -101,6 +101,10 @@ function nullCheck(path, callback) {
   return true;
 }
 
+function isFd(path) {
+  return (path >>> 0) === path;
+}
+
 // Static method to set the stats properties on a Stats object.
 fs.Stats = function(
     dev,
@@ -243,10 +247,18 @@ fs.readFile = function(path, options, callback_) {
     return;
 
   var context = new ReadFileContext(callback, encoding);
+  context.isUserFd = isFd(path); // file descriptor ownership
   var req = new FSReqWrap();
   req.context = context;
   req.oncomplete = readFileAfterOpen;
 
+  if (context.isUserFd) {
+    process.nextTick(function() {
+      req.oncomplete(null, path);
+    });
+    return;
+  }
+
   binding.open(pathModule._makeLong(path),
                stringToFlags(flag),
                0o666,
@@ -257,6 +269,7 @@ const kReadFileBufferLength = 8 * 1024;
 
 function ReadFileContext(callback, encoding) {
   this.fd = undefined;
+  this.isUserFd = undefined;
   this.size = undefined;
   this.callback = callback;
   this.buffers = null;
@@ -293,6 +306,14 @@ ReadFileContext.prototype.close = function(err) {
   req.oncomplete = readFileAfterClose;
   req.context = this;
   this.err = err;
+
+  if (this.isUserFd) {
+    process.nextTick(function() {
+      req.oncomplete(null);
+    });
+    return;
+  }
+
   binding.close(this.fd, req);
 };
 
@@ -374,12 +395,24 @@ function readFileAfterClose(err) {
   else
     buffer = context.buffer;
 
-  if (context.encoding)
-    buffer = buffer.toString(context.encoding);
+  if (err) return callback(err, buffer);
 
-  callback(err, buffer);
+  if (context.encoding) {
+    return tryToString(buffer, context.encoding, callback);
+  }
+
+  callback(null, buffer);
 }
 
+function tryToString(buf, encoding, callback) {
+  var e = null;
+  try {
+    buf = buf.toString(encoding);
+  } catch (err) {
+    e = err;
+  }
+  callback(e, buf);
+}
 
 fs.readFileSync = function(path, options) {
   if (!options) {
@@ -394,7 +427,8 @@ fs.readFileSync = function(path, options) {
   assertEncoding(encoding);
 
   var flag = options.flag || 'r';
-  var fd = fs.openSync(path, flag, 0o666);
+  var isUserFd = isFd(path); // file descriptor ownership
+  var fd = isUserFd ? path : fs.openSync(path, flag, 0o666);
 
   var st;
   var size;
@@ -404,7 +438,7 @@ fs.readFileSync = function(path, options) {
     size = st.isFile() ? st.size : 0;
     threw = false;
   } finally {
-    if (threw) fs.closeSync(fd);
+    if (threw && !isUserFd) fs.closeSync(fd);
   }
 
   var pos = 0;
@@ -419,7 +453,7 @@ fs.readFileSync = function(path, options) {
       buffer = new Buffer(size);
       threw = false;
     } finally {
-      if (threw) fs.closeSync(fd);
+      if (threw && !isUserFd) fs.closeSync(fd);
     }
   }
 
@@ -442,14 +476,15 @@ fs.readFileSync = function(path, options) {
       }
       threw = false;
     } finally {
-      if (threw) fs.closeSync(fd);
+      if (threw && !isUserFd) fs.closeSync(fd);
     }
 
     pos += bytesRead;
     done = (bytesRead === 0) || (size !== 0 && pos >= size);
   }
 
-  fs.closeSync(fd);
+  if (!isUserFd)
+    fs.closeSync(fd);
 
   if (size === 0) {
     // data was collected into the buffers list.
@@ -552,8 +587,8 @@ fs.openSync = function(path, flags, mode) {
 fs.read = function(fd, buffer, offset, length, position, callback) {
   if (!(buffer instanceof Buffer)) {
     // legacy string interface (fd, length, position, encoding, callback)
-    var cb = arguments[4],
-        encoding = arguments[3];
+    const cb = arguments[4];
+    const encoding = arguments[3];
 
     assertEncoding(encoding);
 
@@ -564,10 +599,13 @@ fs.read = function(fd, buffer, offset, length, position, callback) {
 
     callback = function(err, bytesRead) {
       if (!cb) return;
+      if (err) return cb(err);
 
-      var str = (bytesRead > 0) ? buffer.toString(encoding, 0, bytesRead) : '';
-
-      (cb)(err, str, bytesRead);
+      if (bytesRead > 0) {
+        tryToStringWithEnd(buffer, encoding, bytesRead, cb);
+      } else {
+        (cb)(err, '', bytesRead);
+      }
     };
   }
 
@@ -582,6 +620,16 @@ fs.read = function(fd, buffer, offset, length, position, callback) {
   binding.read(fd, buffer, offset, length, position, req);
 };
 
+function tryToStringWithEnd(buf, encoding, end, callback) {
+  var e;
+  try {
+    buf = buf.toString(encoding, 0, end);
+  } catch (err) {
+    e = err;
+  }
+  callback(e, buf, end);
+}
+
 fs.readSync = function(fd, buffer, offset, length, position) {
   var legacy = false;
   var encoding;
@@ -872,29 +920,29 @@ function preprocessSymlinkDestination(path, type, linkPath) {
   }
 }
 
-fs.symlink = function(destination, path, type_, callback_) {
+fs.symlink = function(target, path, type_, callback_) {
   var type = (typeof type_ === 'string' ? type_ : null);
   var callback = makeCallback(arguments[arguments.length - 1]);
 
-  if (!nullCheck(destination, callback)) return;
+  if (!nullCheck(target, callback)) return;
   if (!nullCheck(path, callback)) return;
 
   var req = new FSReqWrap();
   req.oncomplete = callback;
 
-  binding.symlink(preprocessSymlinkDestination(destination, type, path),
+  binding.symlink(preprocessSymlinkDestination(target, type, path),
                   pathModule._makeLong(path),
                   type,
                   req);
 };
 
-fs.symlinkSync = function(destination, path, type) {
+fs.symlinkSync = function(target, path, type) {
   type = (typeof type === 'string' ? type : null);
 
-  nullCheck(destination);
+  nullCheck(target);
   nullCheck(path);
 
-  return binding.symlink(preprocessSymlinkDestination(destination, type, path),
+  return binding.symlink(preprocessSymlinkDestination(target, type, path),
                          pathModule._makeLong(path),
                          type);
 };
@@ -1091,25 +1139,33 @@ fs.futimesSync = function(fd, atime, mtime) {
   binding.futimes(fd, atime, mtime);
 };
 
-function writeAll(fd, buffer, offset, length, position, callback_) {
+function writeAll(fd, isUserFd, buffer, offset, length, position, callback_) {
   var callback = maybeCallback(arguments[arguments.length - 1]);
 
   // write(fd, buffer, offset, length, position, callback)
   fs.write(fd, buffer, offset, length, position, function(writeErr, written) {
     if (writeErr) {
-      fs.close(fd, function() {
-        if (callback) callback(writeErr);
-      });
+      if (isUserFd) {
+        callback(writeErr);
+      } else {
+        fs.close(fd, function() {
+          callback(writeErr);
+        });
+      }
     } else {
       if (written === length) {
-        fs.close(fd, callback);
+        if (isUserFd) {
+          callback(null);
+        } else {
+          fs.close(fd, callback);
+        }
       } else {
         offset += written;
         length -= written;
         if (position !== null) {
           position += written;
         }
-        writeAll(fd, buffer, offset, length, position, callback);
+        writeAll(fd, isUserFd, buffer, offset, length, position, callback);
       }
     }
   });
@@ -1129,16 +1185,27 @@ fs.writeFile = function(path, data, options, callback_) {
   assertEncoding(options.encoding);
 
   var flag = options.flag || 'w';
+
+  if (isFd(path)) {
+    writeFd(path, true);
+    return;
+  }
+
   fs.open(path, flag, options.mode, function(openErr, fd) {
     if (openErr) {
-      if (callback) callback(openErr);
+      callback(openErr);
     } else {
-      var buffer = (data instanceof Buffer) ? data : new Buffer('' + data,
-          options.encoding || 'utf8');
-      var position = /a/.test(flag) ? null : 0;
-      writeAll(fd, buffer, 0, buffer.length, position, callback);
+      writeFd(fd, false);
     }
   });
+
+  function writeFd(fd, isUserFd) {
+    var buffer = (data instanceof Buffer) ? data : new Buffer('' + data,
+        options.encoding || 'utf8');
+    var position = /a/.test(flag) ? null : 0;
+
+    writeAll(fd, isUserFd, buffer, 0, buffer.length, position, callback);
+  }
 };
 
 fs.writeFileSync = function(path, data, options) {
@@ -1153,7 +1220,9 @@ fs.writeFileSync = function(path, data, options) {
   assertEncoding(options.encoding);
 
   var flag = options.flag || 'w';
-  var fd = fs.openSync(path, flag, options.mode);
+  var isUserFd = isFd(path); // file descriptor ownership
+  var fd = isUserFd ? path : fs.openSync(path, flag, options.mode);
+
   if (!(data instanceof Buffer)) {
     data = new Buffer('' + data, options.encoding || 'utf8');
   }
@@ -1170,7 +1239,7 @@ fs.writeFileSync = function(path, data, options) {
       }
     }
   } finally {
-    fs.closeSync(fd);
+    if (!isUserFd) fs.closeSync(fd);
   }
 };
 
@@ -1187,6 +1256,11 @@ fs.appendFile = function(path, data, options, callback_) {
 
   if (!options.flag)
     options = util._extend({ flag: 'a' }, options);
+
+  // force append behavior when using a supplied file descriptor
+  if (isFd(path))
+    options.flag = 'a';
+
   fs.writeFile(path, data, options, callback);
 };
 
@@ -1198,9 +1272,14 @@ fs.appendFileSync = function(path, data, options) {
   } else if (typeof options !== 'object') {
     throwOptionsError(options);
   }
+
   if (!options.flag)
     options = util._extend({ flag: 'a' }, options);
 
+  // force append behavior when using a supplied file descriptor
+  if (isFd(path))
+    options.flag = 'a';
+
   fs.writeFileSync(path, data, options);
 };
 
@@ -1367,18 +1446,14 @@ fs.unwatchFile = function(filename, listener) {
 
 // Regexp that finds the next partion of a (partial) path
 // result is [base_with_slash, base], e.g. ['somedir/', 'somedir']
-if (isWindows) {
-  var nextPartRe = /(.*?)(?:[\/\\]+|$)/g;
-} else {
-  var nextPartRe = /(.*?)(?:[\/]+|$)/g;
-}
+const nextPartRe = isWindows ?
+  /(.*?)(?:[\/\\]+|$)/g :
+  /(.*?)(?:[\/]+|$)/g;
 
 // Regex to find the device root, including trailing slash. E.g. 'c:\\'.
-if (isWindows) {
-  var splitRootRe = /^(?:[a-zA-Z]:|[\\\/]{2}[^\\\/]+[\\\/][^\\\/]+)?[\\\/]*/;
-} else {
-  var splitRootRe = /^[\/]*/;
-}
+const splitRootRe = isWindows ?
+  /^(?:[a-zA-Z]:|[\\\/]{2}[^\\\/]+[\\\/][^\\\/]+)?[\\\/]*/ :
+  /^[\/]*/;
 
 fs.realpathSync = function realpathSync(p, cache) {
   // make p is absolute
@@ -1388,9 +1463,9 @@ fs.realpathSync = function realpathSync(p, cache) {
     return cache[p];
   }
 
-  var original = p,
-      seenLinks = {},
-      knownHard = {};
+  const original = p;
+  const seenLinks = {};
+  const knownHard = {};
 
   // current character position in p
   var pos;
@@ -1490,9 +1565,9 @@ fs.realpath = function realpath(p, cache, cb) {
     return process.nextTick(cb.bind(null, null, cache[p]));
   }
 
-  var original = p,
-      seenLinks = {},
-      knownHard = {};
+  const original = p;
+  const seenLinks = {};
+  const knownHard = {};
 
   // current character position in p
   var pos;
@@ -1763,7 +1838,7 @@ ReadStream.prototype.close = function(cb) {
       this.once('open', close);
       return;
     }
-    return process.nextTick(this.emit.bind(this, 'close'));
+    return process.nextTick(() => this.emit('close'));
   }
   this.closed = true;
   close();
@@ -1807,6 +1882,7 @@ function WriteStream(path, options) {
   this.mode = options.mode === undefined ? 0o666 : options.mode;
 
   this.start = options.start;
+  this.autoClose = options.autoClose === undefined ? true : !!options.autoClose;
   this.pos = undefined;
   this.bytesWritten = 0;
 
@@ -1828,7 +1904,11 @@ function WriteStream(path, options) {
     this.open();
 
   // dispose on finish.
-  this.once('finish', this.close);
+  this.once('finish', function() {
+    if (this.autoClose) {
+      this.close();
+    }
+  });
 }
 
 fs.FileWriteStream = fs.WriteStream; // support the legacy name
@@ -1837,7 +1917,9 @@ fs.FileWriteStream = fs.WriteStream; // support the legacy name
 WriteStream.prototype.open = function() {
   fs.open(this.path, this.flags, this.mode, function(er, fd) {
     if (er) {
-      this.destroy();
+      if (this.autoClose) {
+        this.destroy();
+      }
       this.emit('error', er);
       return;
     }
@@ -1860,7 +1942,9 @@ WriteStream.prototype._write = function(data, encoding, cb) {
   var self = this;
   fs.write(this.fd, data, 0, data.length, this.pos, function(er, bytes) {
     if (er) {
-      self.destroy();
+      if (self.autoClose) {
+        self.destroy();
+      }
       return cb(er);
     }
     self.bytesWritten += bytes;
@@ -1941,9 +2025,9 @@ util.inherits(SyncWriteStream, Stream);
 
 // Export
 Object.defineProperty(fs, 'SyncWriteStream', {
-    configurable: true,
-    writable: true,
-    value: SyncWriteStream
+  configurable: true,
+  writable: true,
+  value: SyncWriteStream
 });
 
 SyncWriteStream.prototype.write = function(data, arg1, arg2) {
