@@ -23,6 +23,20 @@ function hasOwnProperty(obj, prop) {
 }
 
 
+function stat(filename) {
+  filename = path._makeLong(filename);
+  const cache = stat.cache;
+  if (cache !== null) {
+    const result = cache.get(filename);
+    if (result !== undefined) return result;
+  }
+  const result = internalModuleStat(filename);
+  if (cache !== null) cache.set(filename, result);
+  return result;
+}
+stat.cache = null;
+
+
 function Module(id, parent) {
   this.id = id;
   this.exports = {};
@@ -47,7 +61,6 @@ Module.wrapper = NativeModule.wrapper;
 Module.wrap = NativeModule.wrap;
 Module._debug = util.debuglog('module');
 
-
 // We use this alias for the preprocessor that filters it out
 const debug = Module._debug;
 
@@ -105,7 +118,7 @@ Module._realpathCache = {};
 
 // check if the file exists and is not a directory
 function tryFile(requestPath) {
-  const rc = internalModuleStat(path._makeLong(requestPath));
+  const rc = stat(requestPath);
   return rc === 0 && toRealPath(requestPath);
 }
 
@@ -127,28 +140,27 @@ function tryExtensions(p, exts) {
 
 var warned = false;
 Module._findPath = function(request, paths) {
-  var exts = Object.keys(Module._extensions);
-
   if (path.isAbsolute(request)) {
     paths = [''];
   }
 
-  var trailingSlash = (request.slice(-1) === '/');
-
   var cacheKey = JSON.stringify({request: request, paths: paths});
   if (Module._pathCache[cacheKey]) {
     return Module._pathCache[cacheKey];
   }
 
+  const exts = Object.keys(Module._extensions);
+  const trailingSlash = request.slice(-1) === '/';
+
   // For each path
   for (var i = 0, PL = paths.length; i < PL; i++) {
     // Don't search further if path doesn't exist
-    if (paths[i] && internalModuleStat(path._makeLong(paths[i])) < 1) continue;
+    if (paths[i] && stat(paths[i]) < 1) continue;
     var basePath = path.resolve(paths[i], request);
     var filename;
 
     if (!trailingSlash) {
-      const rc = internalModuleStat(path._makeLong(basePath));
+      const rc = stat(basePath);
       if (rc === 0) {  // File.
         filename = toRealPath(basePath);
       } else if (rc === 1) {  // Directory.
@@ -275,17 +287,6 @@ Module._load = function(request, parent, isMain) {
     debug('Module._load REQUEST %s parent: %s', request, parent.id);
   }
 
-  // REPL is a special case, because it needs the real require.
-  if (request === 'internal/repl' || request === 'repl') {
-    if (Module._cache[request]) {
-      return Module._cache[request];
-    }
-    var replModule = new Module(request);
-    replModule._compile(NativeModule.getSource(request), `${request}.js`);
-    NativeModule._cache[request] = replModule;
-    return replModule.exports;
-  }
-
   var filename = Module._resolveFilename(request, parent);
 
   var cachedModule = Module._cache[filename];
@@ -363,7 +364,7 @@ Module.prototype.load = function(filename) {
 Module.prototype.require = function(path) {
   assert(path, 'missing path');
   assert(typeof path === 'string', 'path must be a string');
-  return Module._load(path, this);
+  return Module._load(path, this, /* isMain */ false);
 };
 
 
@@ -377,41 +378,14 @@ var resolvedArgv;
 // the file.
 // Returns exception, if any.
 Module.prototype._compile = function(content, filename) {
-  var self = this;
   // remove shebang
   content = content.replace(shebangRe, '');
 
-  function require(path) {
-    return self.require(path);
-  }
-
-  require.resolve = function(request) {
-    return Module._resolveFilename(request, self);
-  };
-
-  Object.defineProperty(require, 'paths', { get: function() {
-    throw new Error('require.paths is removed. Use ' +
-                    'node_modules folders, or the NODE_PATH ' +
-                    'environment variable instead.');
-  }});
-
-  require.main = process.mainModule;
-
-  // Enable support to add extra extension types
-  require.extensions = Module._extensions;
-  require.registerExtension = function() {
-    throw new Error('require.registerExtension() removed. Use ' +
-                    'require.extensions instead.');
-  };
-
-  require.cache = Module._cache;
-
-  var dirname = path.dirname(filename);
-
   // create wrapper function
   var wrapper = Module.wrap(content);
 
-  var compiledWrapper = runInThisContext(wrapper, { filename: filename });
+  var compiledWrapper = runInThisContext(wrapper,
+                                      { filename: filename, lineOffset: 0 });
   if (global.v8debug) {
     if (!resolvedArgv) {
       // we enter the repl if we're not given a filename argument.
@@ -431,8 +405,14 @@ Module.prototype._compile = function(content, filename) {
       global.v8debug.Debug.setBreakPoint(compiledWrapper, 0, 0);
     }
   }
-  var args = [self.exports, require, self, filename, dirname];
-  return compiledWrapper.apply(self.exports, args);
+  const dirname = path.dirname(filename);
+  const require = internalModule.makeRequireFunction.call(this);
+  const args = [this.exports, require, this, filename, dirname];
+  const depth = internalModule.requireDepth;
+  if (depth === 0) stat.cache = new Map();
+  const result = compiledWrapper.apply(this.exports, args);
+  if (depth === 0) stat.cache = null;
+  return result;
 };
 
 
@@ -472,10 +452,11 @@ Module.runMain = function() {
 Module._initPaths = function() {
   const isWindows = process.platform === 'win32';
 
+  var homeDir;
   if (isWindows) {
-    var homeDir = process.env.USERPROFILE;
+    homeDir = process.env.USERPROFILE;
   } else {
-    var homeDir = process.env.HOME;
+    homeDir = process.env.HOME;
   }
 
   var paths = [path.resolve(process.execPath, '..', '..', 'lib', 'node')];
@@ -498,10 +479,10 @@ Module._initPaths = function() {
   Module.globalPaths = modulePaths.slice(0);
 };
 
-// bootstrap repl
-Module.requireRepl = function() {
-  return Module._load('internal/repl', '.');
-};
+// TODO(bnoordhuis) Unused, remove in the future.
+Module.requireRepl = internalUtil.deprecate(function() {
+  return NativeModule.require('internal/repl');
+}, 'Module.requireRepl is deprecated.');
 
 Module._preloadModules = function(requests) {
   if (!Array.isArray(requests))
