@@ -1,28 +1,10 @@
-// Copyright Joyent, Inc. and other Node contributors.
-//
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the
-// "Software"), to deal in the Software without restriction, including
-// without limitation the rights to use, copy, modify, merge, publish,
-// distribute, sublicense, and/or sell copies of the Software, and to permit
-// persons to whom the Software is furnished to do so, subject to the
-// following conditions:
-//
-// The above copyright notice and this permission notice shall be included
-// in all copies or substantial portions of the Software.
-//
-// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
-// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
-// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
-// USE OR OTHER DEALINGS IN THE SOFTWARE.
+'use strict';
 
-var net = require('net');
-var url = require('url');
-var util = require('util');
-var Buffer = require('buffer').Buffer;
+const net = require('net');
+const url = require('url');
+const binding = process.binding('crypto');
+const Buffer = require('buffer').Buffer;
+const constants = require('constants');
 
 // Allow {CLIENT_RENEG_LIMIT} client-initiated session renegotiations
 // every {CLIENT_RENEG_WINDOW} seconds. An error event is emitted if more
@@ -33,16 +15,12 @@ exports.CLIENT_RENEG_WINDOW = 600;
 
 exports.SLAB_BUFFER_SIZE = 10 * 1024 * 1024;
 
-exports.DEFAULT_CIPHERS =
-    // TLS 1.2
-    'ECDHE-RSA-AES128-SHA256:DHE-RSA-AES128-SHA256:AES128-GCM-SHA256:' +
-    // TLS 1.0
-    'RC4:HIGH:!MD5:!aNULL';
+exports.DEFAULT_CIPHERS = constants.defaultCipherList;
 
 exports.DEFAULT_ECDH_CURVE = 'prime256v1';
 
 exports.getCiphers = function() {
-  var names = process.binding('crypto').getSSLCiphers();
+  const names = binding.getSSLCiphers();
   // Drop all-caps names in favor of their lowercase aliases,
   var ctx = {};
   names.forEach(function(name) {
@@ -54,27 +32,42 @@ exports.getCiphers = function() {
 
 // Convert protocols array into valid OpenSSL protocols list
 // ("\x06spdy/2\x08http/1.1\x08http/1.0")
-exports.convertNPNProtocols = function convertNPNProtocols(NPNProtocols, out) {
-  // If NPNProtocols is Array - translate it into buffer
-  if (util.isArray(NPNProtocols)) {
-    var buff = new Buffer(NPNProtocols.reduce(function(p, c) {
-      return p + 1 + Buffer.byteLength(c);
-    }, 0));
+function convertProtocols(protocols) {
+  var buff = new Buffer(protocols.reduce(function(p, c) {
+    return p + 1 + Buffer.byteLength(c);
+  }, 0));
 
-    NPNProtocols.reduce(function(offset, c) {
-      var clen = Buffer.byteLength(c);
-      buff[offset] = clen;
-      buff.write(c, offset + 1);
+  protocols.reduce(function(offset, c) {
+    var clen = Buffer.byteLength(c);
+    buff[offset] = clen;
+    buff.write(c, offset + 1);
 
-      return offset + 1 + clen;
-    }, 0);
+    return offset + 1 + clen;
+  }, 0);
 
-    NPNProtocols = buff;
+  return buff;
+}
+
+exports.convertNPNProtocols  = function(protocols, out) {
+  // If protocols is Array - translate it into buffer
+  if (Array.isArray(protocols)) {
+    protocols = convertProtocols(protocols);
   }
-
   // If it's already a Buffer - store it
-  if (util.isBuffer(NPNProtocols)) {
-    out.NPNProtocols = NPNProtocols;
+  if (protocols instanceof Buffer) {
+    out.NPNProtocols = protocols;
+  }
+};
+
+exports.convertALPNProtocols = function(protocols, out) {
+  // If protocols is Array - translate it into buffer
+  if (Array.isArray(protocols)) {
+    protocols = convertProtocols(protocols);
+  }
+  // If it's already a Buffer - store it
+  if (protocols instanceof Buffer) {
+    // copy new buffer not to be modified by user
+    out.ALPNProtocols = new Buffer(protocols);
   }
 };
 
@@ -112,12 +105,12 @@ exports.checkServerIdentity = function checkServerIdentity(host, cert) {
     return new RegExp('^' + re + '$', 'i');
   }
 
-  var dnsNames = [],
-      uriNames = [],
-      ips = [],
-      matchCN = true,
-      valid = false,
-      reason = 'Unknown reason';
+  var dnsNames = [];
+  var uriNames = [];
+  const ips = [];
+  var matchCN = true;
+  var valid = false;
+  var reason = 'Unknown reason';
 
   // There're several names to perform check against:
   // CN and altnames in certificate extension
@@ -147,11 +140,9 @@ exports.checkServerIdentity = function checkServerIdentity(host, cert) {
       return ip === host;
     });
     if (!valid) {
-      reason = util.format('IP: %s is not in the cert\'s list: %s',
-                           host,
-                           ips.join(', '));
+      reason = `IP: ${host} is not in the cert's list: ${ips.join(', ')}`;
     }
-  } else {
+  } else if (cert.subject) {
     // Transform hostname to canonical form
     if (!/\.$/.test(host)) host += '.';
 
@@ -180,7 +171,7 @@ exports.checkServerIdentity = function checkServerIdentity(host, cert) {
     // RFC6125
     if (matchCN) {
       var commonNames = cert.subject.CN;
-      if (util.isArray(commonNames)) {
+      if (Array.isArray(commonNames)) {
         for (var i = 0, k = commonNames.length; i < k; ++i) {
           dnsNames.push(regexpify(commonNames[i], true));
         }
@@ -195,21 +186,20 @@ exports.checkServerIdentity = function checkServerIdentity(host, cert) {
 
     if (!valid) {
       if (cert.subjectaltname) {
-        reason = util.format('Host: %s is not in the cert\'s altnames: %s',
-                             host,
-                             cert.subjectaltname);
+        reason =
+            `Host: ${host} is not in the cert's altnames: ` +
+            `${cert.subjectaltname}`;
       } else {
-        reason = util.format('Host: %s is not cert\'s CN: %s',
-                             host,
-                             cert.subject.CN);
+        reason = `Host: ${host} is not cert's CN: ${cert.subject.CN}`;
       }
     }
+  } else {
+    reason = 'Cert is empty';
   }
 
   if (!valid) {
     var err = new Error(
-        util.format('Hostname/IP doesn\'t match certificate\'s altnames: %j',
-                    reason));
+        `Hostname/IP doesn't match certificate's altnames: "${reason}"`);
     err.reason = reason;
     err.host = host;
     err.cert = cert;
@@ -228,7 +218,7 @@ exports.parseCertString = function parseCertString(s) {
       var key = parts[i].slice(0, sepIndex);
       var value = parts[i].slice(sepIndex + 1);
       if (key in out) {
-        if (!util.isArray(out[key])) {
+        if (!Array.isArray(out[key])) {
           out[key] = [out[key]];
         }
         out[key].push(value);
