@@ -1,5 +1,5 @@
 'use strict';
-var common = require('../common');
+require('../common');
 var assert = require('assert');
 
 // If everything aligns so that you do a read(n) of exactly the
@@ -53,9 +53,6 @@ function push() {
     setTimeout(push);
 }
 
-// start the flow
-var ret = r.read(0);
-
 process.on('exit', function() {
   assert.equal(pushes, PUSHCOUNT + 1);
   assert(endEmitted);
